////////////////////////////////////////////////////////////////////////////////
// File         : GraphPrint.h
// Subsystem    : Graph
// Component    : graphs
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class GraphPrint
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 10.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
// This class adds unique numbers to the nodes of a graph and
// allows to print out a graph with a single call of Print().
//     
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_GRAPHPRINT_H__E3FD1D75_319C_4CBE_89BE_E9CA867F0436__INCLUDED_)
#define AFX_GRAPHPRINT_H__E3FD1D75_319C_4CBE_89BE_E9CA867F0436__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "ObjPrint.h"
#include "NodesNumbers.h"
#include "graph.h"

class GraphPrint : public ObjPrint, public NodesNumbers, public Graph  
{
public:
	GraphPrint( const char *name = ObjPrintNoname );
	GraphPrint( const Graph &g, const char *name = ObjPrintNoname);
	GraphPrint( const GraphPrint &g, const char *name = ObjPrintNoname);
	virtual ~GraphPrint();
	GraphPrint& operator=( const GraphPrint &g);
	Node CreateNode(void);	
	Node AddCopyOf(const GraphPrint &g) {Node n = Graph::AddCopyOf( g ); RenumberNew(); return n; };
	Edge AddTransitiveHull( const Node &n1, const Node &n2 ) {Edge e = Graph::AddTransitiveHull( n1, n2 ); RenumberNew(); return e; };
	Edge AddTransitiveHull( const Node &n ) {Edge e = Graph::AddTransitiveHull( n); RenumberNew(); return e; };
	Edge AddTransitiveHull( const Edge &ee ) {Edge e = Graph::AddTransitiveHull( ee ); RenumberNew(); return e; };
	Node AddStar( const Node &n1, const Node &n2 ) {Node n = Graph::AddStar( n1, n2 ); RenumberNew(); return n; }; 
	Node AddStar( const Edge &e ) {Node n = Graph::AddStar( e ); RenumberNew(); return n; };
	Node AddStarBetween( const Node &nn ) {Node n = Graph::AddStarBetween( nn ); RenumberNew(); return n; }; 

private:
	void Copy( const GraphPrint &g);
	void PrintNeighbours( Node &n ) const;
	void PrintBody( void ) const;

	void RenumberNew( void ) { NodesNumbers::RenumberNew( FirstNode() ); };


};


// ****************************** Relation ************************************

class RelationPrint : public ObjPrint, public Relation
{
public:
	RelationPrint( const NodesNumbers &numb, const char *name = ObjPrintNoname ) 
		: ObjPrint( "Relation", name ), m_nodesNumbers( &numb ) {};
	RelationPrint( const NodesNumbers &numb, const Relation &rel, const char *name = ObjPrintNoname ) 
		: ObjPrint( "Relation", name ), Relation( rel ), m_nodesNumbers( &numb ) {};

	void PrintBody( void ) const;

private:
	const NodesNumbers *m_nodesNumbers;
};

// ****************************** NodeVectorPrint ************************************

class NodeVectorPrint : public ObjPrint, public NodeVector 
{
public:
	NodeVectorPrint( const NodesNumbers &numb, const char *name = ObjPrintNoname ) 
		: ObjPrint( "NodeVector", name ), m_nodesNumbers( &numb ) {};
	NodeVectorPrint( const NodesNumbers &numb, const NodeVector &vec, const char *name = ObjPrintNoname ) 
		: ObjPrint( "NodeVector", name ), NodeVector( vec ), m_nodesNumbers( &numb ) {};

	void PrintBody( void ) const;

protected:
	NodeVectorPrint( const NodesNumbers &numb, const char *type, const char *name = ObjPrintNoname ) 
		: ObjPrint( type, name ), m_nodesNumbers( &numb ) {};

private:
	const NodesNumbers *m_nodesNumbers;
};


// ****************************** EdgeVectorPrint ************************************

class EdgeVectorPrint : public ObjPrint, public EdgeVector
{
public:
	EdgeVectorPrint( const NodesNumbers &numb, const char *name = ObjPrintNoname ) 
		: ObjPrint( "EdgeVector", name ), m_nodesNumbers( &numb ) {};
	EdgeVectorPrint( const NodesNumbers &numb, const EdgeVector &vec, const char *name = ObjPrintNoname ) 
		: ObjPrint( "EdgeVector", name ), EdgeVector( vec ), m_nodesNumbers( &numb ) {};

	void PrintBody( void ) const;

protected:
	EdgeVectorPrint( const NodesNumbers &numb, const char *type, const char *name = ObjPrintNoname ) 
		: ObjPrint( type, name ), m_nodesNumbers( &numb ) {};

private:
	const NodesNumbers *m_nodesNumbers;
};

// ****************************** NodePathPrint ************************************

class NodePathPrint : public NodeVectorPrint 
{
public:
	NodePathPrint( const NodesNumbers &numb, const char *name = ObjPrintNoname ) 
		: NodeVectorPrint( numb, "NodePath", name ) { };
	NodePathPrint( const NodesNumbers &numb, const NodePath &vec, const char *name = ObjPrintNoname ) 
		: NodeVectorPrint( numb, vec, name ) {};
};


// ****************************** EdgePathPrint ************************************

class EdgePathPrint : public EdgeVectorPrint
{
public:
	EdgePathPrint( const NodesNumbers &numb, const char *name = ObjPrintNoname ) 
		: EdgeVectorPrint( numb, "EdgePath", name ) { };
};

// ****************************** NodePathVectorPrint ************************************

class NodePathVectorPrint : public ObjPrint, public NodePathVector 
{

private:
	const NodesNumbers *m_nodesNumbers;
};


// ****************************** EdgePathVectoPrintr ************************************

class EdgePathVectorPrint : public ObjPrint, public EdgePathVector
{

private:
	const NodesNumbers *m_nodesNumbers;
};

#endif // !defined(AFX_GRAPHPRINT_H__E3FD1D75_319C_4CBE_89BE_E9CA867F0436__INCLUDED_)
