////////////////////////////////////////////////////////////////////////////////
// File         : HnetUniverse.cpp
// Subsystem    : Graph
// Component    : H-net
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : implementation of the class HnetUniverse
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 17.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
//     
////////////////////////////////////////////////////////////////////////////////


#include "HnetUniverse.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

#ifdef WIN32
#pragma warning( disable : 4355 ) //warning C4355: this' : wird in Initialisierungslisten fuer Basisklasse verwendet
#endif

HnetUniverse::HnetUniverse() : m_isoTypeReferenceCounter( 0 ), 
		m_isoTypeRoot( m_isoTypeGraph.CreateNode() ), m_isoTypeHnet( Hnet( this, m_isoTypeRoot ) )
{
}

#ifdef WIN32
#pragma warning( default : 4355 )  
#endif


HnetUniverse::HnetUniverse( HnetUniverse &universe ) 
	: m_isoTypeHnet( Hnet( &universe ) )
{
	Copy( universe );
}

HnetUniverse::~HnetUniverse()
{

}




//////////////////////////////////////////////////////////////////////
// public
//////////////////////////////////////////////////////////////////////


HnetUniverse & HnetUniverse::operator=( const HnetUniverse &universe)
{
	if (this != &universe)
		Copy( universe );

	return *this;
}


IsoTypeNode HnetUniverse::CreateIsoType( int numOuterEntries, int numOuterExits )
{
	IsoTypeNode isoType = m_isoTypeGraph.CreateNode();
	m_isoTypeReferenceCounter[ isoType ] = 1;
	Hnet &hnet = m_isoTypeHnet[ isoType ];
	hnet = Hnet( numOuterEntries, numOuterExits, this, isoType, false );
	hnet.m_virgin = true; // isoTypeID must change, when something is edited

	// change reference tree for subcenters
	m_isoTypeGraph.CreateEdge( m_isoTypeRoot, isoType );

	return isoType;
}


IsoTypeNode HnetUniverse::CreateIsoTypeElementary( int numOuterEntries, int numOuterExits )
{
	IsoTypeNode isoType = m_isoTypeGraph.CreateNode();
	m_isoTypeReferenceCounter[ isoType ] = 1;
	Hnet &hnet = m_isoTypeHnet[ isoType ];
	hnet = Hnet( numOuterEntries, numOuterExits, this, isoType );
	hnet.m_virgin = true; // isoTypeID must change, when something is edited

	// change reference tree for subcenters
	m_isoTypeGraph.CreateEdge( m_isoTypeRoot, isoType );

	return isoType;
}




IsoTypeNode HnetUniverse::CopyIsoType( const IsoTypeNode &isoTypeOrig )
{
	IsoTypeNode isoType = m_isoTypeGraph.CreateNode();
	m_isoTypeReferenceCounter[ isoType ] = 1;
	m_isoTypeHnet[ isoType ] = m_isoTypeHnet[ isoTypeOrig ];

	// change reference tree for subcenters
	m_isoTypeGraph.CreateEdge( m_isoTypeRoot, isoType );
	Hnet &hnet = m_isoTypeHnet[ isoType ];
	hnet.m_virgin = true; // isoTypeID must change, when something is edited
	hnet.m_isoType[ hnet.m_Snet.Root() ] = isoType;

	Node s = hnet.m_Snet.FirstSubcentre();

	while ( s )
	{
		m_isoTypeGraph.CreateEdgeSingle( isoType, hnet.m_isoType[ s ] );
		m_isoTypeReferenceCounter[ hnet.m_isoType[ s ] ]++;
		s = hnet.m_Snet.NextSubcentre( s );
	}

	return isoType;
}


void HnetUniverse::DelIsoType( IsoTypeNode &isoType )
{
	// delete is only allowed on highest level
	if ( (isoType == m_isoTypeRoot) || (isoType.FirstIngoingEdge().SourceNode() != m_isoTypeRoot ) ) 
		return;

	m_isoTypeGraph.Delete( isoType.FirstIngoingEdge() ); 
	m_isoTypeReferenceCounter[ isoType ]--;
	DelIsoTypeRec( isoType );
}


void HnetUniverse::AddReference( const IsoTypeNode &isoTypeFrom,  const IsoTypeNode &isoTypeTo )
{
	m_isoTypeGraph.CreateEdgeSingle( isoTypeFrom, isoTypeTo );
	m_isoTypeReferenceCounter[ isoTypeTo ]++;
}


void HnetUniverse::DelReference( const IsoTypeNode &isoTypeFrom,  IsoTypeNode &isoTypeTo )
{
	m_isoTypeReferenceCounter[ isoTypeTo ]--;

	if ( m_isoTypeHnet[ isoTypeFrom ].NumberOfReferences( isoTypeTo ) == 0 )
	{
		Edge e = m_isoTypeGraph.CreateEdgeSingle( isoTypeFrom, isoTypeTo ); // just to find the edge
		m_isoTypeGraph.Delete( e ); 

		if ( m_isoTypeReferenceCounter[ isoTypeTo ] == 0) // no references; delete recusively
			DelIsoTypeRec( isoTypeTo );
	}
}


bool HnetUniverse::ValidIsoType( IsoTypeNode &isoTypeNode ) const
{
	IsoTypeNode n = m_isoTypeGraph.FirstNode();

	while ( n )
	{
		if ( n == isoTypeNode )
			return true;
		n = n.NextNode();
	}
	
	return false;
}

//////////////////////////////////////////////////////////////////////
// protected
//////////////////////////////////////////////////////////////////////

#include "GraphPrint.h"

void HnetUniverse::Copy( const HnetUniverse &universe  )
{
	Node n, nOrig;
	NodeAttribute< IsoTypeNode > isoTypeNew;
	IsoTypeNode isoTypeOld;

	m_isoTypeGraph = universe.m_isoTypeGraph;
	m_isoTypeRoot = m_isoTypeGraph.FirstNode();

	// ajust isoTypes
	n = m_isoTypeRoot;
	nOrig = universe.m_isoTypeRoot;

	while ( n && nOrig )
	{
		isoTypeNew[ nOrig ] = n;
		n = n.NextNode();
		nOrig = nOrig.NextNode();
	}

	// copy node attributes
	n = m_isoTypeRoot;
	nOrig = universe.m_isoTypeRoot;

	while ( n && nOrig )
	{
		Hnet &hnet = m_isoTypeHnet[ n ] = universe.m_isoTypeHnet[ nOrig ];

		// ajust isoTypes for hnet

		hnet.m_isoType[ hnet.Root() ] = n;
		hnet.m_universe = this;

		Node s = hnet.FirstSubcentre();

		while ( s )
		{
			hnet.m_isoType[s ] = isoTypeNew[ hnet.m_isoType[ s ] ];
			s = hnet.NextSubcentre( s );
		}


		m_isoTypeReferenceCounter[ n ] = universe.m_isoTypeReferenceCounter[ nOrig ];
		n = n.NextNode();
		nOrig = nOrig.NextNode();
	}
}

//////////////////////////////////////////////////////////////////////
// private
//////////////////////////////////////////////////////////////////////

void HnetUniverse::DelIsoTypeRec( IsoTypeNode &isoType )
{
	Hnet &hnet = m_isoTypeHnet[ isoType ];
	m_isoTypeGraph.Delete( isoType ); // deletes outgoing edges, too!
	Node s = hnet.m_Snet.FirstSubcentre();

	if ( !s )  // elementary H-Net?
	{
		hnet = Hnet( this ); // delete H-net
		return; 
	}

	while ( s )
	{
		if ( --(m_isoTypeReferenceCounter[ hnet.m_isoType[ s ] ]) == 0)
			DelIsoType( hnet.m_isoType[ s ] ); //last reference becomes deleted
		s = hnet.m_Snet.NextSubcentre( s );
	}

	hnet = Hnet(); // delete H-net
}
