////////////////////////////////////////////////////////////////////////////////
// File         : Snet.h
// Subsystem    : Graph
// Component    : S-net
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class Snet
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 10.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
// The class Snet implements s-nets, which are specific graphs.
// For detailed information see the document S-net.doc.
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_SNET_H__79C183FC_D83D_4EA7_AE74_31DE5D11CB58__INCLUDED_)
#define AFX_SNET_H__79C183FC_D83D_4EA7_AE74_31DE5D11CB58__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "graph.h"


typedef enum 
{
	NODE_UNKNOWN, // should not happen
	NODE_SUBCENTRE,
	NODE_ENTRY,
	NODE_EXIT,
} NodeType;


class Snet  
{
public:
	Snet();
	Snet( const Snet &snet) { Copy( snet ); };
	virtual ~Snet();
	Snet& operator=( const Snet &snet);

	// edit operations
	Node AddOuterEntry( void );
	Node AddOuterExit( void );
	Node AddSubcentre( int numEntries, int numExits );
	void ChangeSubcentre( const Node &s, int numEntries, int numExits );
	void DelOuterEntry( Node &y  );
	void DelOuterExit( Node &x  );
	void DelSubcentre( Node &s  );
	Node CopySubcentre( const Node &s ) { return AddSubcentre( NumberOfEntries( s ), NumberOfExits( s ) ); };
	Node CopyAsSubcentre( const Snet &snet ){ return AddSubcentre( snet.NumberOfOuterEntries(), snet.NumberOfOuterExits() ); };
	Node Replace( Node &s, const Snet &snet );
	Edge Connect( const Node &x, const Node &y );
	void Disconnect( Edge &e ); 

	// inquiries
	Node Root( void ) const { return m_g.FirstNode(); }
	Node FirstSubcentre( void ) const; 
	Node NextSubcentre( const Node &n) const;
	Node LastSubcentre( void ) const; 
	Node PrevSubcentre( const Node &n) const; 
	Node SubcentreOf( const Node &n) const; 
	Node FirstEntry( Edge &e, const Node &s ) const { e = s.FirstIngoingEdge(); return e.SourceNode(); }; // on call e is emty an edge
	Node NextEntry( Edge &e ) const { e = e.NextIngoingEdge(); return e.SourceNode(); };
	Node FirstExit( Edge &e, const Node &s ) const { e = s.FirstOutgoingEdge(); return e.TargetNode(); }; // on call e is emty an edge
	Node NextExit( Edge &e ) const { e = e.NextOutgoingEdge(); return e.TargetNode(); };
	int NumberOfEntries( const Node &s ) const { return m_g.NumberOfIngoingEdges( s ); };
	int NumberOfExits( const Node &s ) const { return m_g.NumberOfOutgoingEdges( s ); };
	// Attention! outer entries are inner exits, outer exits are inner entries
	Node FirstOuterEntry( Edge &e) const { e = m_sx.FirstOutgoingEdge(); return e.TargetNode(); }; // on call e is emty an edge
	Node NextOuterEntry( Edge &e ) const { e = e.NextOutgoingEdge(); return e.TargetNode(); };
	Node FirstOuterExit( Edge &e ) const { e = m_sx.FirstIngoingEdge(); return e.SourceNode(); }; // on call e is emty an edge
	Node NextOuterExit( Edge &e ) const { e = e.NextIngoingEdge(); return e.SourceNode(); };
	int NumberOfOuterEntries( void ) const { return NumberOfExits( m_sx ); };
	int NumberOfOuterExits( void ) const { return NumberOfEntries( m_sx ); };
	Edge FirstConnection( void ) const; 
	Edge NextConnection( const Edge &e) const;
	NodeType GetNodeType( const Node &n ) const { return m_nodeType[ n ]; }

	// additional functions
	void MakeTransitive( void ) { while (MakeTransitiveRec()); };
	void TransComplement( Snet &complement, NodeAttribute< Node > &originNode ); // creates the transitive complement
	bool ComputeRelations( NodeAttribute< int > &color, NodeAttribute< bool > &multible, NodeAttribute< Relation > &relation );
	int MarkComponents( NodeAttribute< int > &componentNumber );
	void DeleteUnreachableNodes( NodeAttribute< int > &color, int defaultColor, int invalidColor );


protected:
	Graph m_g;
	Node  m_sx; //marked subcenter of closure
	mutable NodeAttribute< NodeType > m_nodeType;

protected:
	void Copy( const Snet &snet);

private:
	bool MakeTransitiveRec( void );
	void SetNodeTypes( const Node &s ); // set m_nodeType for a subcentre and its entries and exits
	void MarkNeighbours( Node n, NodeAttribute< int > &componentNumber, int componentNr ); 

};

#endif // !defined(AFX_SNET_H__79C183FC_D83D_4EA7_AE74_31DE5D11CB58__INCLUDED_)
